'use strict'
var _ = require('lodash');


//相关操作----------------------------------------------

/**
 * 数组根据长度均匀拆分  
 * @param {Array<string|number|object>} 数组
 * @param {number} 拆分长度
 * @returns {Array} 返回一个数组
 * _.chunk(['a', 'b', 'c', 'd'], 2)  => [['a', 'b'], ['c', 'd']] 
 */
exports.chunk = function (array, num) {
    return _.chunk(array, num);
}

/**
 * 深度拷贝，数组内部对象重新建立 
 * @param {object} 对象
 * @returns {Array} 返回一个新对象
 */
exports.clone = function (objects) {
    return _.cloneDeep(objects);
}

/**
 * 从左开始删除数组，num为个数，默认为1 
 * @param {Array} 数组
 * @param {number} 删除个数
 * @returns {Array} 返回删除后的数组
 * array=[1,2,3,4,5]
 * num=1 return [2,3,4,5]
 * num=3 return [4,5]
 */
exports.drop = function (array, num) {
    return _.drop(array, num);
}


/**
 * 从右开始删除数组，num为个数，默认为1 
 * @param {Array} 数组
 * @param {number} 删除个数
 * @returns {Array} 返回删除后的数组
 * array=[1,2,3,4,5]
 * num=1 return [1,2,3,4]
 * num=3 return [1,2]
 */
exports.dropRight = function (array, num) {
    return _.dropRight(array, num);
}

/**
 * 异或，去除相同的，然后去重，所有的比较仅和第一个传参比较
 * 支持[value,object]
 * [1, 2, 6], [1, 2, 3, 4, 5] => [6, 3, 4, 5]
 * @param {...Array<string|number|object>}
 * @returns {Array} 返回一个数组
 */
exports.xor = function () {
    var args = [];
    [].push.apply(args, arguments);
    args.push(_.isEqual)
    return _.xorWith.apply(_.xorWith, args)
}

/**
 * 不包含，a不在b中，以第后一位为中心
 * 支持[value,object]
 * [1, 2, 6],[5],[1, 2, 3, 4, 5]=> [3, 4]
 * @param {...Array(string|number|object)}
 * @returns {Array} 返回一个数组
 */
exports.notIn = function () {
    var args = [];
    [].push.apply(args, arguments);
    args.push(_.isEqual)
    return _.differenceWith.apply(_.differenceWith, args)
}

/**
 * 合并数组
 * @param {Array} array The array to concatenate.
 * @param {...*} [values] The values to concatenate.
 * @returns {Array} Returns the new concatenated array.
 * _.concat(array, 2, [3], [[4]]) => [1, 2, 3, [4]]
 */
exports.concat = function () {
    var objs = arguments;
    return _.concat.apply(_.concat, objs);
}

/**
 * 去除数组里空值
 * [0, 1, false, 2, '', 3] => [1, 2, 3]
 * @param {Array} array The array to compact.
 * @returns {Array} Returns the new compact array.
 */
exports.compact = function (arr) {
    return _.compact(arr);
}

/**
 * 把数组变平
 * @param {Array} array The array to concatenate.
 * @return {Array} Returns the new flatten array.
 *  [1, [2, [3, [4]], 5]]=> [1, 2, [3, [4]], 5]
 */
exports.flatten = function (arr) {
    return _.flatten(arr);
}

/**
 * 遍历集合运算成功数组
 * function square(n) {return n * n;}
 * map([4, 8], square); => [16, 64]
 * @param  {array} objects
 * @param  {function} fn 可以是迭代函数
 * @return 返回执行迭代函数后的数组
 */
exports.map = function (objects, fn) {
    return _.map(objects, fn);
}

/**
 * 获取集合里某个key对应的value数组
 * values([{ 'user': 'barney' }, { 'user': 'fred' }],'user') => ['barney', 'fred']
 * values({ 'user': 'barney' }) => ['barney']
 * @param  {Array|Object} objects  对象集合
 * @param  {String} key 允许为空
 * @return {Array} 返回key对应的value数组
 */
exports.values = function (objects, key) {
    return _.map(objects, key);
}

/**
 * 去重
 * [{n:1},{n:2},{n:1}] => [{n:1},{n:2}]
 * [1,2,2,3] => [1,2,3]
 * @param  {array} arr [number|object]
 * @return  {array}
 */
exports.uniq = function (arr) {
    if (!arr) return [];
    if (!Array.isArray(arr) || arr.length == 0) return [];
    if (_.isPlainObject(arr[0])) return _.uniqWith(arr, _.isEqual);
    return _.uniq(arr);
}

//返回数组中位置如[1,2,3]找2，return 1;
exports.indexOf = function (arr, value) {
    return _.indexOf(arr, value);
}

//查找对象数组的位置,return int
exports.findIndex = function (objects, obj) {
    if (typeof obj == 'string' || typeof obj == 'number') return objects.indexOf(obj);
    return _.findIndex(objects, obj);
}

// //[{ 'n': 1 }, { 'n': 2 }]; find {n:2}
// //{ 'n': 2 }
// //找到一条就返回
// exports.findOne = function (objects, obj) {
//     return _.find(objects, obj);
// }
// //[{ 'n': 1 }, { 'n': 2 }]; find {n:2}|[{n:1},{n:2}]
// //[{n:1},{n:2}]
// //遍历所有行
// exports.find = function (objects, cons) {
//     if (!Array.isArray(objects)) return [];
//     if (!cons || !_.isObject(cons)) return objects;
//
//     if (_.isPlainObject(cons)) {
//         return _.filter(objects, cons);
//     }
//     var arr = [];
//     cons = _.uniqWith(cons, _.isEqual);
//     cons.forEach(function (c) {
//         var _arr = _.filter(objects, c);
//         arr = arr.concat(_arr);
//     })
//     return arr;
// }
/**
 * 从集合找到符合条件的一条记录
 * @param  {array} objects　[{a:{b:1,e:{f:6}},c:2},{c:2},{d:5}]
 * @param  {object} con {'a.b':1}
 * @return  {object} 返回单条记录或者null
 */
exports.findOne = function (objects, con) {
    if (!Array.isArray(objects)) return [];
    let arrCon = [];
    for (let key in con) {
        arrCon.push({ key, value: con[key] })
    }
    if (arrCon.length == 0) return objects;
    for (let obj of objects) {
        if (checkCon(obj, arrCon)) return obj;
    }
    return null;
}
/**
 * 从集合找到符合条件的一条记录
 * @param  {array} objects　[{a:{b:1,e:{f:6}},c:2},{c:2},{d:5}]
 * @param  {object} con {'a.b':1}
 * @return  {array}　返回数组
 */
exports.find = function (objects, con) {
    if (!Array.isArray(objects)) return [];
    let arrCon = [];
    for (let key in con) {
        arrCon.push({ key, value: con[key] })
    }
    if (arrCon.length == 0) return objects;
    let arr = [];
    for (let obj of objects) {
        if (checkCon(obj, arrCon)) arr.push(obj);
    }
    return arr;
}
function checkCon(obj, arrCon) {
    for (let con of arrCon) {
        if (con.key.indexOf('.') == -1) {
            //不带. key
            if (obj[con.key] !== con.value) return false;
        } else {
            //带. key
            let arr = con.key.split('.');
            let val = obj;
            for (let item of arr) {
                if (!val.hasOwnProperty(item)) return false;
                val = val[item];
            }
            if (val !== con.value) return false;
        }
    }
    return true;
}

/**
 * 最大值
 * max([4, 2, 8, 6])=>8
 * max([{a:1},{a:2}],'a')=>{a:2}
 * @param  {array} arr [number|object]
 * @param  {string|function} key? 允许为空
 * @return  {number|object}
 */
exports.max = function (arr, key) {
    return _.maxBy(arr, key);
}

/**
 * 最小值
 * min([4, 2, 8, 6])=>2
 * min([{a:1},{a:2}],'a')=>{a:1}
 * @param  {array} arr [number|object]
 * @param  {string|function} key? 允许为空
 * @return  {number|object}
 */
exports.min = function (arr, key) {
    return _.minBy(arr, key);
}

/**
 * 根据key进行排序，支持纯数组和集合
 * @param  {array} arr [number,number]|[object,object]
 * @param  {string|array} keys?  允许为空可以是'a'|['a',b]
 */
exports.sort = function (arr, keys) {
    return _.sortBy(arr, keys);
}

/**
 * 数组随机，不改原来数组
 * @param  {array} arr
 * @return  {array} 新数组
 */
exports.sortRandom = function (arr) {
    if (!arr || arr.length == 0) return [];
    var arrN = exports.clone(arr);
    arrN.sort(function () {
        return Math.random() - 0.5;
    })
    return arrN;
}

/**
 * 把集合转化成以key为主键的对象,kv键值对
 * [{id:"key1",b:2},{id:"key2",b:3}]=>{"key1":{id:"key1",b:2},"key2":{id:"key2",b:3}}
 * @param  {array} arr
 * @param  {string|function} key 也可以转入fn
 */
exports.kv = function (arr, key) {
    return _.keyBy(arr, key);
}

