var fs = require("fs");
var filesystem = require("./filesystem");

var archiver = require("archiver");
var unzipper = require("unzipper");

var path = require("path");

/**
 * 压缩文件
 * @param  {string} src 压缩目录
 * @param  {string} dist 目标文件/xx/test.zip
 * @param  {object} opts 选填 {ignore:[]//忽略某些文件}
 * @return  {boolean}
 */
//opts {ignore: ["**/node_modules/**", "**/ *.log", "** /tmp/ **", "**/*.zip"]}
exports.zip = function (src, dist, opts) {
    return new Promise(function (resolve, reject) {
        opts = opts || {};
        src = path.normalize(src);
        var dir = path.dirname(dist);
        filesystem.mkdir(dir);
        var output = fs.createWriteStream(dist);
        var archive = archiver("zip", {
            zlib: { level: 9 }, // Sets the compression level.
        });

        output.on("close", function () {
            console.log(`文件压缩完成:${archive.pointer()}bytes`);
            resolve(true);
        });

        output.on("end", function () {
            // console.log("Data has been drained");
            resolve(true);
        });

        // good practice to catch warnings (ie stat failures and other non-blocking errors)
        archive.on("warning", function (err) {
            if (err.code === "ENOENT") {
                // log warning
            } else {
                // throw error
                console.error(err);
                resolve(false);
                // throw err;
            }
        });

        // good practice to catch this error explicitly
        archive.on("error", (err) => {
            //throw err;
            console.error(err);
            resolve(false);
        });

        // pipe archive data to the file
        archive.pipe(output);

        // archive.glob(["**", "**/.**"], {
        //     // pattern:'.**',
        //     cwd: src,
        //     //ignore: ["**/node_modules/**", "**/*.log", "**/tmp/**", "**/*.zip"],
        // });
        opts.cwd = src;
        archive.glob(["**", "**/.**"], opts);
        archive.finalize();
    });
}

/**
 * 解压文件
 * @param  {string} src 解压文件 /xx/test.zip
 * @param  {string} dist 目标目录
 * @return  {boolean}
 */
exports.unzip = function (src, dist) {
    return filesystem.mkdir(dist).then(function (res) {
        return fs.createReadStream(src).pipe(unzipper.Extract({ path: dist })).promise();
    }).then(function (res) {
        return true;
    }).catch(function (err) {
        console.error('解压失败', err)
        return false;
    });
}


